import { Batch } from "@/app/admin/batches/variables/batch";
import { createFileRequest } from "@/app/lib/kyc/action";
import { ImageUpload } from "@/types/attachment";
import { PaginationData } from "@/types/pagination";
import { IbcInterface } from "@/types/settings/ibc/ibc";
import { UserInterface } from "@/types/user/user";

export const getStatusColors = (status: string) => {
  switch (status) {
    case "offline":
      return {
        background: "#b0bec5",
        borderColor: "#546e7a",
        glyphColor: "#78909c",
      };
    case "online":
      return {
        background: "#0f9d58",
        borderColor: "#006425",
        glyphColor: "#60d98f",
      };
    case "traveling":
      return {
        background: "#ff9800",
        borderColor: "#e65100",
        glyphColor: "#ffcc80",
      };
    case "arrived":
      return {
        background: "#00c853",
        borderColor: "#009624",
        glyphColor: "#69f0ae",
      };
    case "attempting":
      return {
        background: "#ff6d00",
        borderColor: "#e65100",
        glyphColor: "#ffab40",
      };
    case "break":
      return {
        background: "#d32f2f",
        borderColor: "#b71c1c",
        glyphColor: "#ff8a80",
      };
    default:
      return {
        background: "#9e9e9e",
        borderColor: "#616161",
        glyphColor: "#bdbdbd",
      };
  }
};

export const generateColor = (index: number) =>
  `hsl(${(index * 137.5) % 360}, 70%, 50%)`;

export const extractPagination = (data: any): PaginationData => {
  return {
    total: data.total,
    per_page: data.per_page,
    current_page: data.current_page,
    last_page: data.last_page,
    next_page_url: data.next_page_url,
    prev_page_url: data.prev_page_url,
    links: data.links.map(
      (link: { label: string; url: string | null; active: boolean }) => ({
        label: link.label,
        url: link.url,
        active: link.active,
      })
    ),
  };
};

export const base64toFile = (imageSrc: string) => {
  if (imageSrc) {
    // Convert base64 to Blob
    const byteString = atob(imageSrc.split(",")[1]);
    const mimeString = imageSrc.split(",")[0].split(":")[1].split(";")[0];
    const ab = new ArrayBuffer(byteString.length);
    const ia = new Uint8Array(ab);
    for (let i = 0; i < byteString.length; i++) {
      ia[i] = byteString.charCodeAt(i);
    }
    const blob = new Blob([ab], { type: mimeString });
    // Create a File object from the Blob
    const file = new File([blob], "image.png", { type: mimeString });
    // Create a FormData object and append the File
    return file;
  }
};

const onFileRequest = async (image: File) => {
  const formData = new FormData();
  formData.append("file", image);

  const res = await createFileRequest(formData);

  return res;
};

export const onUploadFiles = async (images: ImageUpload[]) => {
  const results = await Promise.all(
    images.map(async (image) => {
      try {
        const res = await onFileRequest(image.file);
        return res?.data;
      } catch (error) {
        console.error(`Error processing image:`, error);
        return null; // Handle errors gracefully
      }
    })
  );
  return results
    .filter((result) => result !== null)
    ?.map((result) => result?.id); // Filter out failed uploads if necessary
};

// Function to generate a unique ID
export const generateUniqueId = () => {
  return `${Date.now()}-${Math.floor(Math.random() * 10000)}`;
};

export const onCompleteImage = async (params: any) => {
  const results = await Promise.all(
    params?.map(async (image: string) => {
      try {
        const res = await onFileRequest(base64toFile(image));
        return res?.data;
      } catch (error) {
        console.error(`Error processing image:`, error);
        return null; // Handle errors gracefully
      }
    })
  );
  return results
    .filter((result) => result !== null)
    ?.map((result) => result?.id); // Filter out failed uploads if necessary
};

export const downloadFile = async (
  fileId: number,
  fileName: string,
  token: string
) => {
  try {
    // Call the API to download the file
    const path = `v1/files/force-download/${fileId}`;
    const baseUrl = `${process.env.NEXT_PUBLIC_API_BASE_URL}${path}`;
    const response = await fetch(baseUrl, {
      method: "GET",
      headers: {
        Authorization: `Bearer ${token}`,
        Accept: "application/octet-stream",
      },
    });

    if (!response.ok) {
      throw new Error("Failed to download file");
    }

    // Convert response to blob
    const blob = await response.blob();
    const url = window.URL.createObjectURL(blob);

    // Create a link and trigger the download
    const a = document.createElement("a");
    a.href = url;
    a.download = fileName; // Use provided file name
    document.body.appendChild(a);
    a.click();
    a.remove();

    // Clean up
    window.URL.revokeObjectURL(url);
  } catch (error) {
    throw error;
  }
};

export const prepareUsersDropdown = (users: { data: UserInterface[] }) => {
  return users.data && users.data.length > 0
    ? users.data.map((user) => ({
        label: `${user.first_name} ${user.last_name ? user.last_name : ""}`,
        value: user.id,
      }))
    : [];
};

export const prepareIbcDropdown = (ibcs: { data: IbcInterface[] }) => {
  return ibcs.data && ibcs.data.length > 0
    ? ibcs.data.map((ibc: IbcInterface) => ({
        label: `${ibc.area_name}`,
        value: ibc.id,
      }))
    : [];
};

export const prepareBatchesDropdown = (batches: Batch[]) => {
  return batches && batches.length > 0
  ? batches.map((batch) => ({
    label: `${batch.name}`,
    value: String(batch.id),
  }))
  : [];
}
